package com.bwie.auth.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.bwie.auth.feign.UserFeignService;
import com.bwie.auth.service.AuthService;
import com.bwie.common.constants.JwtConstants;
import com.bwie.common.constants.TokenConstants;
import com.bwie.common.domain.User;
import com.bwie.common.domain.request.LoginRequest;
import com.bwie.common.domain.response.JwtResponse;
import com.bwie.common.exception.ServiceException;
import com.bwie.common.result.Result;
import com.bwie.common.utils.JwtUtils;
import com.bwie.common.utils.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import java.util.HashMap;
import java.util.Map;
import java.util.UUID;
import java.util.concurrent.TimeUnit;

/**
 * @ProjectName: week3-1129
 * @PackageName: com.bwie.auth.service.impl
 * @Description TODO
 * @Author LiYonJie
 * @Date 2023/11/29
 * @Version 1.0
 */
@Service
public class AuthServiceImpl implements AuthService {

    @Autowired
    private UserFeignService userFeignService;

    @Autowired
    private RedisTemplate<String, String> redisTemplate;

    @Autowired
    private HttpServletRequest request;

    /**
     * 1.1 项目需要统一处理异常信息，不能直接抛出异常（2分）
     * @param loginRequest
     * @return
     * @throws ServiceException
     */
    @Override
    public Result<JwtResponse> login(LoginRequest loginRequest) throws ServiceException{
        if (StringUtils.isEmpty(loginRequest.getUsername()) || StringUtils.isEmpty(loginRequest.getPassword())){
            return Result.error("用户名密码不能为空");
        }
        Result<User> byName = userFeignService.findByName(loginRequest.getUsername());
        User loginUser = byName.getData();
        if (null == loginUser){
            throw new ServiceException("该用户不存在");
        }
        if (!loginUser.getPassword().equals(loginRequest.getPassword())){
            throw new ServiceException("密码错误");
        }
        // 1.3 鉴权使用JWT + Redis完成（5分）
        String userKey = UUID.randomUUID().toString().replaceAll("-", "");
        Map<String, Object> jwtMap = new HashMap<>(50, 500);
        jwtMap.put(JwtConstants.USER_KEY, userKey);
        jwtMap.put(JwtConstants.DETAILS_USER_ID, loginUser.getUserId());
        String token = JwtUtils.createToken(jwtMap);
        redisTemplate.opsForValue().set(TokenConstants.LOGIN_TOKEN_KEY + userKey, JSONObject.toJSONString(loginUser),
                30, TimeUnit.MINUTES);
        JwtResponse jwtResponse = new JwtResponse();
        jwtResponse.setToken(token);
        jwtResponse.setExpireTime("30MIN");
        return Result.success(jwtResponse);
    }

    @Override
    public User userInfo() {
        String token = request.getHeader(TokenConstants.TOKEN);
        String userKey = JwtUtils.getUserKey(token);
        String json = redisTemplate.opsForValue().get(TokenConstants.LOGIN_TOKEN_KEY + userKey);
        return JSONObject.parseObject(json, User.class);
    }

    @Override
    public void logout() {
        String token = request.getHeader(TokenConstants.TOKEN);
        String userKey = JwtUtils.getUserKey(token);
        redisTemplate.hasKey(TokenConstants.LOGIN_TOKEN_KEY + userKey);
    }
}
