package com.bwie.common.exception;

import com.bwie.common.result.Result;
import lombok.extern.log4j.Log4j2;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.RestControllerAdvice;

import javax.servlet.http.HttpServletRequest;
import java.io.IOException;

/**
 * @ClassName:
 * @Description:  全局异常处理器
 * @Author: LiYonJie
 * @Date: 2023/11/28
 */
@RestControllerAdvice
@Log4j2
public class GlobalExceptionHandler extends Throwable {

    private static final long serialVersionUID = 1L;

    /**
     * 错误码
     */
    private Integer code;

    /**
     * 错误提示
     */
    private String message;

    /**
     * 错误明细，内部调试错误
     * <p>
     */
    private String detailMessage;

    /**
     * 空构造方法，避免反序列化问题
     */
    public void IOException() {
    }

    public void IOException(String message) {
        this.message = message;
    }

    public void IOException(String message, Integer code) {
        this.message = message;
        this.code = code;
    }

    public String getDetailMessage() {
        return detailMessage;
    }

    public GlobalExceptionHandler setDetailMessage(String detailMessage) {
        this.detailMessage = detailMessage;
        return this;
    }

    @Override
    public String getMessage() {
        return message;
    }

    public GlobalExceptionHandler setMessage(String message) {
        this.message = message;
        return this;
    }

    public Integer getCode() {
        return code;
    }


    /**
     * 处理  IO流异常
     * @return
     */
    @ExceptionHandler(IOException.class)
    public Result ioExceptionHandler(IOException ioException,HttpServletRequest request) {
        String requestURI = request.getRequestURI();
        log.error("请求路径:【{}】,IO流处理异常:【{}】",requestURI,ioException.getMessage());
        return Result.error(Result.FAIL, ioException.getMessage());
    }

}
