package com.bwie.scenic.rabbitmq;

import com.bwie.common.domain.Order;
import com.bwie.common.domain.Scenic;
import com.bwie.common.exception.GlobalExceptionHandler;
import com.bwie.scenic.mapper.OrderMapper;
import com.bwie.scenic.service.ScenicService;
import com.rabbitmq.client.Channel;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.rabbit.annotation.Queue;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.io.IOException;

import static com.bwie.scenic.service.impl.OrderServiceImpl.REFUND_QUEUE;

/**
 * @ProjectName: week3-1129
 * @PackageName: com.bwie.scenic.rabbitmq
 * @Description TODO
 * @Author LiYonJie
 * @Date 2023/11/29
 * @Version 1.0
 */
@Component
@Slf4j
public class RabbitMqConfig {

    @Autowired
    private RedisTemplate<String, String> redisTemplate;

    @Autowired
    private OrderMapper orderMapper;

    @Autowired
    private ScenicService scenicService;

    /**
     * 1.6 异步完成退票功能。消息保证不丢失不重复消费
     *
     * @param orderId
     * @param message
     * @param channel
     */
    @Transactional
    @RabbitListener(queuesToDeclare = {@Queue(name = REFUND_QUEUE)})
    public void refund(Integer orderId, Message message, Channel channel) throws GlobalExceptionHandler {
        String messageId = message.getMessageProperties().getMessageId();
        Long add = redisTemplate.opsForSet().add(messageId, REFUND_QUEUE);
        log.info("消息队列:【{}】,接受到信息:【{}】,开始消费...", REFUND_QUEUE, orderId);
        try {
            if (add.intValue() > 0) {
                // 获取当前订单信息
                Order order = orderMapper.findById(orderId);

                // 获取景区信息
                Scenic scenic = scenicService.findById(order.getScenicId());
                // 景区剩余票数+已退票数
                scenic.setRemainingVotes(scenic.getRemainingVotes() + order.getVotes());
                scenicService.updateRemainingVotes(scenic);

                // 订单为已退票
                order.setStatues(3);
                orderMapper.updateOrder(order);

                channel.basicAck(message.getMessageProperties().getDeliveryTag(), false);
                log.info("消息队列:【{}】,接受到信息:【{}】,消费成功...", REFUND_QUEUE, orderId);
            } else {
                log.info("消息队列:【{}】,接受到信息:【{}】,消费重复...", REFUND_QUEUE, orderId);
            }
        } catch (IOException e) {
            throw new GlobalExceptionHandler();
        }
    }
}
