package com.bwie.scenic.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.bwie.common.constants.TokenConstants;
import com.bwie.common.domain.Order;
import com.bwie.common.domain.Scenic;
import com.bwie.common.domain.User;
import com.bwie.common.domain.request.OrderRequest;
import com.bwie.common.domain.response.OrderResponse;
import com.bwie.common.utils.JwtUtils;
import com.bwie.scenic.mapper.OrderMapper;
import com.bwie.scenic.service.OrderService;
import com.bwie.scenic.service.ScenicService;
import org.springframework.amqp.AmqpException;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.core.MessagePostProcessor;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletRequest;
import java.math.BigDecimal;
import java.util.Date;
import java.util.List;
import java.util.UUID;
import java.util.concurrent.TimeUnit;

/**
 * @ProjectName: week3-1129
 * @PackageName: com.bwie.scenic.service.impl
 * @Description TODO
 * @Author LiYonJie
 * @Date 2023/11/29
 * @Version 1.0
 */
@Service
public class OrderServiceImpl implements OrderService {

    @Autowired
    private OrderMapper orderMapper;

    @Autowired
    private ScenicService scenicService;

    @Autowired
    private RedisTemplate<String, String> redisTemplate;

    @Autowired
    private HttpServletRequest request;

    @Autowired
    private RabbitTemplate rabbitTemplate;

    public static final String REFUND_QUEUE = "refund";

    @Transactional
    @Override
    public void buy(OrderResponse orderResponse) {
        // 获取用户信息
        String token = request.getHeader(TokenConstants.TOKEN);
        String userKey = JwtUtils.getUserKey(token);
        String json = redisTemplate.opsForValue().get(TokenConstants.LOGIN_TOKEN_KEY + userKey);
        User user = JSONObject.parseObject(json, User.class);

        // 获取景区信息
        Scenic scenic = scenicService.findById(orderResponse.getScenicId());

        orderResponse.setUserId(user.getUserId());
        orderResponse.setPayCard(user.getBankCard());
        orderResponse.setScenicId(scenic.getScenicId());
        orderResponse.setPayTime(new Date());
        orderMapper.buy(orderResponse);

        // 1.5 购票后从所选银行卡扣除余额
        user.setBalance(user.getBalance().subtract(orderResponse.getTotalFares()));
        orderMapper.updateUserBalance(user);
        redisTemplate.opsForValue().set(TokenConstants.LOGIN_TOKEN_KEY + userKey, JSONObject.toJSONString(user),
                30, TimeUnit.MINUTES);

        // 1.5 景区剩余票数-已购票数
        scenic.setRemainingVotes(scenic.getRemainingVotes() - orderResponse.getVotes());
        scenicService.updateRemainingVotes(scenic);
    }

    /**
     * 1.6 展示已购所有票信息，展示成功
     * @param orderRequest
     * @return
     */
    @Override
    public List<OrderResponse> list(OrderRequest orderRequest) {
        return orderMapper.list(orderRequest);
    }

    @Transactional
    @Override
    public void refund(Integer orderId) {
        rabbitTemplate.convertAndSend(REFUND_QUEUE, orderId, new MessagePostProcessor() {
            @Override
            public Message postProcessMessage(Message message) throws AmqpException {
                message.getMessageProperties().setMessageId(UUID.randomUUID().toString().replaceAll("-",""));
                return message;
            }
        });

        // 获取用户信息
        String token = request.getHeader(TokenConstants.TOKEN);
        String userKey = JwtUtils.getUserKey(token);
        String json = redisTemplate.opsForValue().get(TokenConstants.LOGIN_TOKEN_KEY + userKey);
        User user = JSONObject.parseObject(json, User.class);

        // 获取当前订单信息
        Order order = orderMapper.findById(orderId);

        // 1.6 钱款退回原路，扣除10%的手续费，退票成功
        user.setBalance(user.getBalance().add(order.getTotalFares().multiply(new BigDecimal(0.9))));
        orderMapper.updateUserBalance(user);
        redisTemplate.opsForValue().set(TokenConstants.LOGIN_TOKEN_KEY + userKey, JSONObject.toJSONString(user),
                30, TimeUnit.MINUTES);
    }
}
